//
//  AppDelegate.swift
//  Lecture15_TimerAppBasicStudents
//
//  Created by Hamza on 6/09/18.
//  Copyright © 2018 Hamza. All rights reserved.
//

import Cocoa

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate, TimerModelDelegate {

    @IBOutlet weak var window: NSWindow!

   // Outlet to the label that displays the formatted timer value
    @IBOutlet weak var outletLabel : NSTextField!
    
    // Outlets to the buttons at the bottom of the window
    @IBOutlet weak var outletReset : NSButton!
    @IBOutlet weak var outletStartStop : NSButton!
       // Outlets to the menus that we added
    @IBOutlet weak var outletStartStopMenu : NSMenuItem!
    @IBOutlet weak var outletResetMenu : NSMenuItem!
    
    var timer: TimerModel? = nil ;

    
    
    // Toggle the state of the timer
    // (startTimer will create NSTimer if needed)
    @IBAction func actionStartStop (_ sender : Any) {
        if ( timer!.running == true ){
            stopTimer ();
        } else { // running == false !!
            startTimer ();
        }
    }
     // The reset action zeros the counter
    @IBAction func actionReset (_ sender : Any) {
        timer!.reset();
    }
    
    // Update the label to show the current time, formatted for display
    // callback target from TimerModel
    func secondsChanged(_ seconds: Int) {
        // Take the total number of seconds the timer has run for
        var secondsLocal = seconds ;
          // Determine how many hours and minutes this represents
        let hour = secondsLocal/3600;
        secondsLocal %= 3600;
        let min = secondsLocal/60;
        secondsLocal %= 60;
        // Update the label with the formatted timer value
        outletLabel . stringValue = String (
            format :"%02ld:%02ld:%02ld",
            hour ,min , secondsLocal );
    }
    
    

    func startTimer(){
        timer!.start();
        // Update the Start/Stop button label and menu and state of the reset button
        outletStartStop . title = " Stop ";
        outletStartStopMenu . title = " Stop ";
        outletReset . isEnabled = false ;
    }

    func stopTimer(){
        timer!.stop();
          // Update the Start/Stop button label/Menu and state of the reset button
        outletReset . isEnabled = true ;
        outletStartStop . title = " Start ";
        outletStartStopMenu . title = " Start ";
    }


    
    // When users interact with a menu, the system checks whether items should be greyed out.
    // We use our "running" state to determine whether to grey out start/Stop or Reset.
    override func validateMenuItem(_ menuItem: NSMenuItem) -> Bool {
        if ( menuItem == outletStartStopMenu){
            return true;
        } else if (menuItem == outletResetMenu){
            return !timer!.running;
        } else {
            return true;
        }
    }
    
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        // Insert code here to initialize your application
        timer = TimerModel();
        timer!.delegate = self;
        timer!.reset();
    }

    func applicationWillTerminate(_ aNotification: Notification) {
        // Insert code here to tear down your application
    }


}

