//
//  TimerModel.swift
//  Lecture15_TimerAppBasicStudents
//
//  Created by Hamza on 6/09/18.
//  Copyright © 2018 Hamza. All rights reserved.
//

import Foundation


// classes implement this protocol to receive callbacks from the TimerModel
public protocol TimerModelDelegate {
    func secondsChanged(_ second: Int);
}

public class TimerModel {
    
    // Instance variables to maintain counter state:
    // - the number of seconds that the timer has run for:
    private var seconds : Int = 0 ;
    // going to stop
    private var stopping : Bool = false;
    public fileprivate(set) var running : Bool = false ;
    
    public var delegate: TimerModelDelegate? = nil;
    
    func alertDelegate(){
              // if a delegate isn't defined, do nothing
        delegate?.secondsChanged(seconds);
    }
    
    // Callback method invoked by the timer each second it is running
    @objc func countUp (_ theTimer : Foundation . Timer ){
        if ( !stopping == true ){
            // If our timer is running, update the seconds counter
            self . seconds += 1;
            // ... and update the label
            alertDelegate();
        } else {
            // If our timer is not running, stop this OS timer
            theTimer . invalidate ();
            stopping = false;
            running = false;
        }
    }
    
    
    // Start the timer
    func start (){
        if (running == true) { return ; }
        // Create a timer object that calls the countUp method every second
        let theTimer = Foundation.Timer ( timeInterval :
            1, target : self , selector :
            #selector ( TimerModel . countUp (_:)) ,
               userInfo : nil , repeats : true );
        // Get a reference to the current event loop
        let loop = RunLoop . current ;
        // Attach the above timer to the event loop:
        // it will then actually start firning
        loop .add( theTimer , forMode :
            RunLoopMode . commonModes );
        // Change our internal state to running
        self . running = true ;
        
    }
    
    // Stop the timer
    func stop () {
        // We are not running the timer anymore
        // ... but the actual NSTimer won't be cleaned up until it next fires.
        // This NSTimer clean-up occurs in the countUp method above.
        self . running = false ;
        stopping = true;
        
    }
    func reset (){
        self . seconds = 0;
        alertDelegate();
    }
        
    
}
